import os
import commands
from subprocess import call
import glob
import sys

def make_runFiles(
        ideal_number,
        startFile,
        jobName):
    """ 
    Make ideal_number number of copies of the run file with changes to LAMBDANR and job-name
    input:
        ideal_number: number of lambda values you need made, it will make run files for 0 to ideal_number-1 changing the LAMBDANR to match the label on the file
        startFile: This should be a template for the run scripts you want built. The copies created will be split at the . and a lambda number will be added
            for example run.sh as an example script will yield run.0.sh, run.1.sh, and so on
        jobName: This string will be filled into the variable --job-name in the run scripte followed by _lambda for each lambda
    """
    # Split file name
    filePieces = startFile.split('.')
    # make a new file for each lambda value
    for n in range(ideal_number):
        f = open(startFile, "r")
        # Get lambda from file name run.#.sh
        lines = f.readlines()
        f.close()
        new = []
        # iterate through lines
        for l in lines:
            #Change job name
            if "--job-name" in l:
                new.append("#SBATCH --job-name=\"%s_%s\"\n" % (jobName, str(n)))
            # Change lambda value
            elif "export LAMBDANR" in l:
                new.append("export LAMBDANR=%s\n" % str(n))
            # otherwise add line as is to output
            else:
                new.append(l)
        # Create and write new file
        f2 = open("%s.%s.sh" % (filePieces[0], str(n)), "w")
        f2.writelines(new)
        f2.close()

def make_MDPFiles(ideal_number, MDP_change = 'XXX', baseName = "*.X.mdp", letter = 'X'):
    """
    Assuming there is a set of template mdp files it makes copies for all lambda values
    input:
        ideal_number: number of lambda values
        MDP_change: This is a string that is in your mdp file where the lambda value needs to be. This string must not exist anywhere else in your mdp file, such as XXX
        baseName: This should be the name for your mdp files, it can have wild cards to get all mdp files you need, but it must have a letter than can be replaced with the lambda value, for example minimize.X.mdp. 
        letter: letter in the file names that will be replaced with the lambda number
    """
    # get all template mdp files
    files = glob.glob(baseName)
    for elem in files:
        # Read in template file:
        f = elem 
        fil = open(f, 'r')
        s = fil.read()
        # Make a copy for all the lambda numbers
        for i in range(ideal_number):
            of = f.replace(letter, str(i))
            # Replace the MDP_change with the lambda number for that file
            oss = s.replace(MDP_change, str(i))
            # Write the output file
            with open(of, 'w') as outputfile:
                outputfile.write(oss)


def getFiles(
        MDP_path, 
        place_path,
        jobName = "jobName",
        ideal_number = 20,
        startFile = "run.X.sh",
        rewriteRun = True,
        rewriteMDP = True,
        MDP_change = "XXX",
        baseName =  "*.X.mdp",
        letter = 'X'):
    """    
    Uses methods above to get copies of run and/or mdp files for all lambda numbers
    This will move all files in MDP_path to place_path so you only want templates of things you want moved in your MDP_path
        MDP_path = location for all MDP_files and run files
        place_path = where you want to copy them
        rewriteRun = boolean do you want run files made (doesn't overwrite anything)
        rewriteMDP = boolean, do you want mdp files made (doesn't overwrite anything)
        input for the 'make_MDPFiles' and 'make_runFiles' methods
    """
    # Current place so the script will return back
    current_path = os.getcwd() 
    # Copy all files from MDP_path to current_path
    commands.getoutput("cp %s/* %s" % (MDP_path, place_path))

    # change to directory where you want to change your run files
    os.chdir(place_path)

    if rewriteRun:
        make_runFiles(ideal_number, startFile, jobName)

    if rewriteMDP:
        make_MDPFiles(ideal_number, MDP_change)
    # Return to current directory
    os.chdir(current_path)
