import os
from scipy import stats
import sys
import pickle as p
import numpy as np
from pylab import *
import matplotlib.patches as patches
import tools

def makePlot(non, alco, lowLim, highLim, XLabel, YLabel, figName):
    # Expecting data to come in the form [calculated, experimental, dcalc] for each non and alcohols
    # Assign experimental error
    nonErr = [0.30 for i in range(len(non[0]))]
    alcoErr = [0.30 for i in range(len(alco[0]))]
    # From David's script, meets ACS style guide requirements
    wid = 3.25
    #height = wid * (sqrt(5.0) - 1.0) / 2.0
    height = wid
    parameters = {'backend': 'ps', 'axes.labelsize': 6,'text.fontsize': 6,'xtick.labelsize': 5,'ytick.labelsize': 5,'figure.figsize': [wid, height], 'legend.fontsize': 5, 'font.family':'sans-serif', 'font.sans-serif':'arial' }
    legendsize = '5'
    rcParams.update(parameters)

    # Initialize figure
    fig = figure(1)
    ylim(lowLim, highLim)
    xlim(lowLim, highLim)
    ax = fig.add_subplot(111)

    # Add Axes Labels
    xlabel(XLabel)
    ylabel(YLabel)

    # plot x = y line
    L = np.array([lowLim - 20, highLim +20])
    # Add shaded area
    ax.fill_between(L, L+1.5, L-1.5, facecolor = 'wheat', alpha = 0.5, linewidth = 0.0)
    yellow = patches.Patch(color = 'wheat', label = r'$\pm$ 1.5 log units')
    ax.plot(L, L, 'k-', markersize = 0.5, linewidth = 0.01)

    # Plot non-alcohols
    nplt = ax.errorbar(non[0], non[1], xerr = non[2], yerr = nonErr, fmt = 'r^', label = 'non-alcohols', markersize = 3.0, capsize = 0.5, elinewidth = 0.25)
    # Plot alcohols
    aplt = ax.errorbar(alco[0], alco[1], xerr = alco[2], yerr = alcoErr, fmt = 'ko', mfc = 'white', label = 'alcohols', markersize = 3.0, capsize = 0.5, elinewidth = 0.25)
    
        
    # Create Legend
    ax.legend(bbox_to_anchor = (0.02, 0.98), loc = 2, ncol = 1, borderaxespad = 0.0, handles = [nplt, aplt, yellow])
    subplots_adjust(left = 0.1, right = 0.95, bottom = 0.15, top = 0.95)
    savefig("%s.eps" % figName)
    savefig("%s.pdf" % figName)
    close('all')

def makeAxe(ax, non, alco, lowLim, highLim, XLabel, YLabel, title, Arrow = None):
    # Expecting data to come in the form [calculated, experimental, dcalc, dexp] for each non and alcohols
    # Add setting to axis
    ax.set_xlim(lowLim, highLim)
    ax.set_xlabel(XLabel)
    ax.set_ylim(lowLim, highLim)
    ax.set_ylabel(YLabel)
    ax.set_title(title)
    # plot x = y line
    L = np.array([lowLim - 20, highLim +20])
    # Add shaded area
    ax.fill_between(L, L+1.5, L-1.5, facecolor = 'wheat', alpha = 0.5, linewidth = 0.0)
    yellow = patches.Patch(color = 'wheat', label = r'$\pm$ 1.5 log units')
    ax.plot(L, L, 'k-', markersize = 0.5, linewidth = 0.01)

    # Plot non-alcohols
    nplt = ax.errorbar(non[1], non[0], xerr = non[3], yerr = non[2], fmt = 'r^', label = 'non-alcohols', markersize = 3.0, capsize = 0.5, elinewidth = 0.25)
    # Plot alcohols
    aplt = ax.errorbar(alco[1], alco[0], xerr = alco[3], yerr = alco[2], fmt = 'ko', mfc = 'white', label = 'alcohols', markersize = 3.0, capsize = 0.5, elinewidth = 0.25)
    
    # Create Legend
    ax.legend(bbox_to_anchor = (0.98, 0.02), loc = 4, ncol = 1, borderaxespad = 0.0, handles = [nplt, aplt, yellow])
    if not Arrow == None:
        for a in Arrow:
            ax.annotate(a[0], xy = (a[2], a[1]), xytext = (a[4], a[3]), fontsize = 5, arrowprops=dict(arrowstyle="->", connectionstyle="arc3,rad=.01", linewidth = 0.25))
            #arrowprops = dict(arrowstyle = "-> ", connentionstyle = "arc3,rad=0.3") 
    return ax

def makeFourInOnePlot(non1, non2, non3, non4, alco1, alco2, alco3, alco4, lowLim, highLim, XLabel, YLabel, titles, figName, Arrows):
    # Expecting data to come in the form [calculated, experimental, dcalc, dexp] for each non and alcohols
    # Assign experimental error
    # From David's script, meets ACS style guide requirements
    wid = 7.0
    height = wid * (sqrt(5.0) - 1.0) / 2.0
    #height = wid
    parameters = {'backend': 'ps', 'axes.labelsize': 6,'text.fontsize': 6,'xtick.labelsize': 5,'ytick.labelsize': 5,'figure.figsize': [wid, height], 'legend.fontsize': 5, 'font.family':'sans-serif', 'font.sans-serif':'arial' }
    legendsize = '5'
    rcParams.update(parameters)

    # Initialize figure
    fig = figure(1)
    # Use makeAxe method to make the subplots with the appropriate data
    ax1 = makeAxe(fig.add_subplot(221), non1, alco1, lowLim[0], highLim[0], XLabel[0], YLabel[0], titles[0], Arrows[0])
    ax2 = makeAxe(fig.add_subplot(222), non2, alco2, lowLim[1], highLim[1], XLabel[1], YLabel[1], titles[1], Arrows[1])
    ax3 = makeAxe(fig.add_subplot(223), non3, alco3, lowLim[2], highLim[2], XLabel[2], YLabel[2], titles[2], Arrows[2])
    ax4 = makeAxe(fig.add_subplot(224), non4, alco4, lowLim[3], highLim[3], XLabel[3], YLabel[3], titles[3], Arrows[3])

    # Adjust subplots and save big figure    
    # subplots_adjust(left = 0.1, right = 0.95, bottom = 0.15, top = 0.95)
    savefig("%s.eps" % figName)
    savefig("%s.pdf" % figName)
    close('all')

# Expecting data to come in the form [calculated, experimental, dcalc] for each non and alcohols
def getData(d, nonKey, alcoKey, solv, lowLim, highLim, XLabel, YLabel, figName):
    s = solv.split('_')[0]
    # Collect Exp Data (no uncertainty)
    nonExp = [d[k][s]['LogP_exp'][0] for k in nonKey]
    dnonExp = [0.30 for k in nonKey]
    alcoExp = [d[k][s]['LogP_exp'][0] for k in alcoKey]
    dalcoExp = [0.030 for k in alcoKey]
    # Collect Calc Data 
    nonCalc = [d[k][solv]['LogP_calc'][0] for k in nonKey]
    dnonCalc = [d[k][solv]['LogP_calc'][1] for k in nonKey]
    alcoCalc = [d[k][solv]['LogP_calc'][0] for k in alcoKey]
    dalcoCalc = [d[k][solv]['LogP_calc'][1] for k in alcoKey]
    # Organize data into array and remove Nones
    nonData = tools.RemoveNones(np.array([nonCalc, nonExp, dnonCalc, dnonExp]))
    alcoData = tools.RemoveNones(np.array([alcoCalc, alcoExp, dalcoCalc, dalcoExp]))

    # Make plots
    makePlot(nonData, alcoData, lowLim, highLim, XLabel, YLabel, figName)
    return nonData, alcoData

def doErrorAnalysis(d, bootits, dataAt = 'data', Noise = True):
    """ This uses methods in the tools module to do error analysis on any dataset in the dictionary. 
    It returns a copy of the same dictionary with the statistics added to each key

    d = dictionary 
    bootits = number of bootstrap iterations
    dataAt = key where the data is stored in each embedded dictionary
    Where ever the data is stored it is assumed to come in the form of [calc, exp, dcalc, dexp] 
    Noise = boolean indicates if experimental noise should be used during bootstrapping. 
    """
    data = d.copy()
    # Get error analysis for every set of data in dictionary
    for k, e in data.items():
        e['ASE'], e['RMSE'], e['AUE'], e['tau'], e['R'], e['maxE'], e['Correct Sign'], e['R2'] = tools.stats_array(e[dataAt][0], e[dataAt][1], e[dataAt][3], bootits, sid = k, noise = Noise)

    return data

def makeTexTable(d, keys, fileName, caption, tableLabel, dont = {}):
    """
    Makes a table with columns labeled with keys 
    doesn't include rows with data in the dont dictionary
    """
    output = ['\\begin{center}\n', '\\begin{table}\n']
    # Make tabular line
    output.append('\\begin{tabular}{|' + 'c|'*len(keys)+'}\n')
    output.append('\\hline\n')

    # Make column labels
    cols = ''
    for i, k in enumerate(keys):
        # If its the last entry, don't use & and put an end line
        if i == len(keys) - 1:
            cols += '{\\bf %s} \\\\\n' % k
        else: # Otherwise add column label and put & to distinguish separation
            cols += '{\\bf %s} & ' % k

    output.append(cols)
    output.append('\\hline\n')

    for k, e in d.items():
        skip = False
        # If the key, entry combination is in the dont dictionary skip this entry
        for label, entry in dont.items():
            if e[label] in entry:
                skip = True
        if skip:
            continue
        
        line = ""

        for i, key in enumerate(keys):
            # for now if it is a list it needs the uncertainties included in the table
            if type(e[key]) == list:
               data = '$ %.3f \\pm %.3f $' % (e[key][0], e[key][1])
            else:
                data = e[key]

            if i == len(keys) - 1:
                line += data + '\\\\ \n'
            else:
                line += data + ' & '

        output.append(line)
        output.append('\\hline\n')
    
    output.append('\\end{tabular}\n')
    output.append('\\caption{\\label{%s} %s}\n' % (tableLabel, caption))
    output.append('\\end{table}\n')
    output.append('\\end{center}\n')

    f = open(fileName, 'w')
    f.writelines(output)
    f.close()

# ==============================================================================================================================
# Import data from database file and get statistics

database = p.load(open('../Data/dictionary_allResults.p','rb'))
alcohols = p.load(open('../Data/alcohols.p','rb'))
nonKey = []
alcoKey = []
for k, e in database.items():
    if e['number'] == 'str':
        continue
    elif e['number'] in alcohols:
        alcoKey.append(k)
    else: # Not alcohol
        nonKey.append(k)

# Get alcohol and non-alcohol data for both solvents and both FF
cycNon, cycAlco = getData(database, nonKey, alcoKey, 'cyclohexane', -4.0, 5.5, r'Experimental $\log P_{cyc}$', r'Calculated $\log P_{cyc}$', '../extraFigures/cycGAFF')
cycDCnon, cycDCalco = getData(database, nonKey, alcoKey, 'cyclohexane_DC', -4.0, 5.5, r'Experimental $\log P_{cyc}$', r'Calculated $\log P_{cyc}$', '../extraFigures/cycGAFF_DC')
octNon, octAlco = getData(database, nonKey, alcoKey, 'octanol', -1.2, 8.1, r'Experimental $\log P_{cyc}$', r'Calculated $\log P_{cyc}$', '../extraFigures/octGAFF')
octDCnon, octDCalco = getData(database, nonKey, alcoKey, 'octanol_DC', -1.2, 8.1,r'Experimental $\log P_{cyc}$', r'Calculated $\log P_{cyc}$', '../extraFigures/octGAFF_DC')

# Setting for big plot
highLimits = [5.5, 5.5, 8.1, 8.1]
lowLimits =[-4.0, -4.0, -1.2, -1.2]  
XLabel = ['', '', r'Experimental $\log P_{cyc}$', r'Experimental $\log P_{oct}$']
#YLabel = [r'Experimental $\log P_{cyc}$', r'Experimental $\log P_{cyc}$',  r'Experimental $\log P_{oct}$', r'Experimental $\log P_{oct}$']
YLabel = [r'Calculated $\log P$', '', r'Calculated $\log P$', '']
#XLabel = [r'Calculated $\log P_{cyc}$', r'Calculated $\log P_{cyc}$', r'Calculated $\log P_{oct}$', r'Calculated $\log P_{oct}$']
titles = [r'a. Cyclohexane/water $\log P$ with GAFF', r'b. Cyclohexane/water $\log P$ with GAFF-DC', r'c. Octanol/water $\log P$ with GAFF', r'd. Octanol/water $\log P$ with GAFF-DC']
Arrows = [None, None, None, None]

#Make combined plot without labels
makeFourInOnePlot(cycNon, cycDCnon, octNon, octDCnon, cycAlco, cycDCalco, octAlco, octDCalco, lowLimits, highLimits, XLabel, YLabel, titles, '../extraFigures/AllPlots', Arrows) 

# Make "arrow" lists for labeling outliers
cycAr = [['1307', 5.04, -0.52, 4.0, -2.4], ['1879', 3.84, -0.10, 2.2, -0.8]]
cycDCAr = [['1307', 4.0, -0.52, 3.0, -2.4], ['1879', 3.84, -0.10, 2.2, -0.8]]
octAr = [['5785', 6.30, 2.48, 6.8, 1.3]]
octDCAr = [['5785', 4.5, 2.45, 5.25, 1.3]]
Arrows = [cycAr, cycDCAr, octAr, octDCAr]

# Make combined plot with labels
makeFourInOnePlot(cycNon, cycDCnon, octNon, octDCnon, cycAlco, cycDCalco, octAlco, octDCalco, lowLimits, highLimits, XLabel, YLabel, titles, '../Paper/ComparisonPlots', Arrows) 

# =========================================================================================================
# Now combine data and get all of the statistics 
# Uses methods from the tools module and bootstrapping to do error analysis

# Number of iterations for bootstrapping
bootits = 1000 

# Make lists to combine data into all possible sets
cycGAFF = np.concatenate((cycNon, cycAlco), axis = 1)
cycGAFF_DC = np.concatenate((cycDCnon, cycDCalco), axis = 1)
octGAFF = np.concatenate((octNon, octAlco), axis = 1)
octGAFF_DC = np.concatenate((octDCnon, octDCalco), axis = 1)
All = np.concatenate((cycNon, cycAlco, octNon, octAlco), axis = 1)
All_DC = np.concatenate((cycDCnon, cycDCalco, octDCnon, octDCalco), axis = 1)

# Make a dictionary to store stastics in 
stats_dict = {'AllGAFF': {'data': All, 'Solvent': 'both', 'Force Field': 'GAFF', 'Subset': 'All'},
        'AllGAFF_DC': {'data': All_DC, 'Solvent': 'both', 'Force Field': 'GAFF-DC', 'Subset': 'All'},
        'cycGAFF': {'data': cycGAFF, 'Solvent': 'cyclohexane', 'Force Field': 'GAFF', 'Subset': 'All' },
        'cycGAFF_DC': {'data': cycGAFF_DC,'Solvent': 'cyclohexane', 'Force Field': 'GAFF-DC', 'Subset': 'All' }, 
        'cycNonGAFF': {'data': cycNon, 'Solvent': 'cyclohexane', 'Force Field': 'GAFF', 'Subset': 'non-alcohols' },
        'cycAlcoGAFF': {'data': cycAlco, 'Solvent': 'cyclohexane', 'Force Field': 'GAFF', 'Subset': 'Alcohols' },
        'cycNonGAFF_DC': {'data': cycDCnon, 'Solvent': 'cyclohexane', 'Force Field': 'GAFF_DC', 'Subset': 'non-alcohols' },
        'cycAlcoGAFF_DC': {'data': cycDCalco, 'Solvent': 'cyclohexane', 'Force Field': 'GAFF_DC', 'Subset': 'Alcohols' },  
        'octGAFF': {'data': octGAFF, 'Solvent': 'octanol', 'Force Field': 'GAFF', 'Subset': 'All' },
        'octGAFF_DC': {'data': octGAFF_DC,'Solvent': 'octanol', 'Force Field': 'GAFF-DC', 'Subset': 'All' }, 
        'octNonGAFF': {'data': octNon, 'Solvent': 'octanol', 'Force Field': 'GAFF', 'Subset': 'non-alcohols' },
        'octAlcoGAFF': {'data': octAlco, 'Solvent': 'octanol', 'Force Field': 'GAFF', 'Subset': 'Alcohols' },
        'octNonGAFF_DC': {'data': octDCnon, 'Solvent': 'octanol', 'Force Field': 'GAFF_DC', 'Subset': 'non-alcohols' },
        'octAlcoGAFF_DC': {'data': octDCalco, 'Solvent': 'octanol', 'Force Field': 'GAFF_DC', 'Subset': 'Alcohols' }
        }

# Perform error analysis for all data subsets
stats_dict = doErrorAnalysis(stats_dict,bootits)

# Get features for tex table for the error analysis. 
keys = ['Solvent', 'Force Field', 'Subset', 'R', 'RMSE', 'ASE', 'Correct Sign']
caption = "Shown are the Pearson correlation coefficient (R), the root-mean-squared error (RMSE), average signed error (ASE), and percent correct sign for the comparison of calculated and experimental partition coefficients in both organic solvents and the selected force fields. Analysis is presented for the entire set of solutes (All) and separately for only hydroxyl containing solutes (Alcohols)."
tableLabel = "Statistics"
dont = {'Solvent': ['both'], 'Subset': ['non-alcohols']}
dont = {}
makeTexTable(stats_dict, keys, '../extraFigures/Stats_testing.tex', caption, tableLabel, dont)

p.dump(stats_dict, open('errorAnalysis.p','wb'))

# Use SciPy to calculate ttest 
for a,b,lab in [(All[0], All_DC[0], 'all'), (cycGAFF[0], cycGAFF_DC[0], 'cyc'), (octGAFF[0], octGAFF_DC[0], 'oct')]:
    t, p = stats.ttest_rel(a,b)
    print "%s: t: %.4f, p: %.4e" % (lab, t, p)

