#================================================================================
# Created to find differences in experimental partition coefficients when
# there were multiple logP values provided for the same solute in
# Leo, Albert et. al. Chem Rev. 71 (1971) 525-616
#================================================================================

import pickle as p
import numpy as np

# load dictionary
data = p.load(open('dictionary_allResults.p','rb'))
cycDif = []
octDif = []
# find all entries with multiple logP values for cyclohexane and octanol
for k, e in data.items():
    num = e['number']
    if num == 'str':
        continue
    # get experimental numbers
    cycList = e['cyclohexane']['LogP_exp']
    octList = e['octanol']['LogP_exp']

    # If there is more than one, find the max difference and add to dif list
    if len(cycList) > 1:
        print "%s (%s) cyc" % (k, num)
        cycDif.append(np.abs(np.max(cycList) - np.min(cycList)))

    # Repeat for octanol list
    if len(octList) > 1:
        print "%s (%s) oct" % (k, num)
        octDif.append(np.abs(np.max(octList) - np.min(octList)))

# We want averages for the P not logP, so find P for each item in list
cycDifExp = np.array([10**c for c in cycDif])
octDifExp = np.array([10**o for o in octDif])
# Combine exponential lists
allExp = np.array([10**p for p in cycDif+octDif])

# Take the log of the averages
cycAve = np.log10(np.average(cycDifExp))
octAve = np.log10(np.average(octDifExp))
allAve = np.log10(np.average(allExp))

# Print out data
print "Averages from exponents:"
print "oct: %.4f cyc: %.4f all %.4f" % (octAve, cycAve, allAve)

print "Averages from log differences:"
print "oct: %.4f cyc: %.4f all: %.4f" % (np.average(octDif), np.average(cycDif), np.average(octDif + cycDif))

