# Written by Caitlin C. Bannan 
# Fall 2015
# It uses Alchemical Analysis results to calculate logP
# Sometimes in a less than efficient way, comments to come

import numpy as np
import commands
import sys
import pickle as p
import os

database = p.load(open("../Data/dictionary_noResults.p","rb"))
resultsData = {}
Warning_Limit = 3
kB = 1.380648*6.02214129/1000.0
conv = 4.184
warnings = []
result_File = "../Data/%s/%s/results.pickle"
solvent_list = ['cyclohexane','water','octanol']

alcohols = p.load(open('../Data/alcohols.p','rb'))

def getEnergy(fileName, Warning_Limit = 2.):
    warn = None
    if not os.path.isfile(fileName):
        split = fileName.split('/')
        print "\t file not found for", split[1], " in ", split[2], fileName
        warn = "File not found" 
        return None, None, warn

    getData = p.load(open(fileName, 'rb'))
    T = getData.temperature

    dF = getData.dFs[2]
    ddF = getData.ddFs[2]

    energies = np.array([e for (k, e) in dF.items()])
    uncertanties = np.array([e for (k, e) in ddF.items()])

    MBAR = np.array([-dF['MBAR'], ddF['MBAR']])
        
    maxDif = max(energies) - min(energies)
    maxUn = max(uncertanties)
    tol = maxUn*Warning_Limit
    if (np.abs(maxDif) > (tol)) and (tol > 1):
        warn = "Warning energy difference (%.3f) is bigger than %s * the largest uncertainty (%.3f)" % (maxDif, str(Warning_Limit), maxUn)

    kJ = MBAR * kB * T
    return kJ[0],kJ[1],warn

for (molname, entry) in database.items():
    if molname == 'TestKeys':
        continue
    num = entry['number']    
    print num, molname
    for solv in solvent_list:
        En, Un, warn = getEnergy(result_File % (num, solv))
        if warn != None:
            warnings.append(warn + " %s in %s" % (num, solv))
            entry[solv]['warn'] = warn
        entry[solv]['energy'] = [En, Un]
    En, Un, warn = getEnergy(result_File % (num+'_DC', 'octanol'))
    if warn != None:
        warnings.append(warn + "%s_DC in octanol" % num)
        entry['octanol_DC']['warn'] = warn
    database[molname]['octanol_DC']['energy'] = [En, Un]
    if num in alcohols:
        En, Un, warn = getEnergy(result_File % (num+'_DC', 'cyclohexane'))
        if warn != None:
            warnings.append(warn+" %s_DC in cyclohexane" % num)
            entry['cyclohexane_DC']['warn'] = warn
        database[molname]['cyclohexane_DC']['energy'] = [En, Un]
        En, Un, warn = getEnergy(result_File % (num+'_DC', 'water'))
        if warn != None:
            warnings.append(warn+" %s_DC in water" % num)
            entry['water_DC']['warn'] = warn
        database[molname]['water_DC']['energy'] = [En, Un]    
    else: # If num is not an alcohol then the solvation free energy with GAFF-DC would the the same as GAFF in water and cyclohexane
        watDCen = database[molname]['water']['energy'] 
        database[molname]['water_DC']['energy'] = watDCen
        cycDCen =database[molname]['cyclohexane']['energy'] 
        database[molname]['cyclohexane_DC']['energy'] = cycDCen
for (molename, entry) in database.items():
    num = entry['number']
    print num, molename
    if molename == 'TestKeys':
        continue
    if not entry['water'].has_key('energy'):
        continue
    h2o_en = entry['water']['energy']
    for solv in ['cyclohexane','octanol']:
        try:
            solv_en = entry[solv]['energy']
            if solv_en[0] == None:
                print '\t no', solv, 'results', solv_en
                continue
            if h2o_en[0] == None:
                print '\t no water results'
                continue
        except:
            continue
        Dif = np.array([(h2o_en[0] - solv_en[0]), np.sqrt(h2o_en[1]**2 + solv_en[1]**2)])
        LogP = Dif/(np.log(10.)*kB*298.15)
        entry[solv]['LogP_calc'] = LogP
        perErrList = []
        if not None in entry[solv]['LogP_exp']:
            for exp_logP in entry[solv]['LogP_exp']:
                if exp_logP == 0.0:
                    perErrList.append(0.0)
                else:
                    perErrList.append(np.abs((LogP[0]-exp_logP)/exp_logP))
            entry[solv]['perError'] = perErrList
        else:
            print "\tNo Experimental Data for ", num, "in", solv
    
    h2oDC_en = entry['water_DC']['energy']
    if h2oDC_en[0] != None:
        h2o_en = h2oDC_en

    for solv in ['cyclohexane_DC','octanol_DC']:
        try:
            solv_en = entry[solv]['energy']
            if solv_en[0] == None:
                if (solv == 'cyclohexane_DC' and num in alcohols) or solv == 'octanol_DC':
                    print '\t no', solv, 'results'
                continue
            elif h2o_en[0] == None:
                print '\t no water_DC results'
                continue
        except:
            if solv == 'octanol_DC':
                print "no DC results for", num, "in", solv
            elif num in alcohols:
                print 'no DC results for', num, 'in', solv
            continue
        Dif = np.array([(h2o_en[0] - solv_en[0]), np.sqrt(h2o_en[1]**2 + solv_en[1]**2)])
        LogP = Dif/(np.log(10.)*kB*298.15)
        entry[solv]['LogP_calc'] = LogP
        perErrList = []
        refSolv = solv.split('_')[0]
        if not None in entry[refSolv]['LogP_exp']:
            for exp_logP in entry[refSolv]['LogP_exp']:
                if exp_logP == 0.0:
                    perErrList.append(0.0)
                else:
                    perErrList.append(np.abs((LogP[0]-exp_logP)/exp_logP))
        entry[solv]['perError'] = perErrList
            
p.dump(database, open('../Data/dictionary_allResults.p','wb'))
