# Adapted by Caitlin C Bannan June 2015
# Original version written by David L Mobley
# reads data from csv file with experimental data and stores in a dictionary that is saved as a pickle file 

from openeye.oechem import * #Basic OEChem toolkits
from openeye.oeomega import * #Conformation generation
from openeye.oeiupac import * #Chemical naming
import pickle

#Read in CSV table from Caitlin with all the compounds she pulled from Hansch tables
file = open('../Data/ExpData.csv', 'r')
text = file.readlines()
file.close()

#We will parse the file into a set of cyclohexane partitioning coefficients stored in a dictionary keyed by compound name. Values and references will be stored in lists (as in some cases there are multiples).
# csv file is sorted by solvent so all cyclohexane data will be added first. 


#Set up storage for dictionary
database = {}

#Parse file
for line in text[1:]: # First line is column headers 
    #Parse line
    tmp = line.strip().split(',')
    number = tmp[0]
    solvent = tmp[1]
    reference = tmp[2]
    LogP = float(tmp[3])
    name = tmp[4]

    #Otherwise, it is cyclohexane (assumed) so store
    if solvent.lower() == 'cyclohexane':
        #If it is not already in our database, add it
        if not database.has_key(name):
            # Add key and place holders for cyclohexane and cyclohexane_DC 
            database[name] = { 'number':number}
            database[name]['cyclohexane'] =  {'ref':[reference], 'LogP_exp':[LogP], 'LogP_calc': [None, None], 'energy': [None, None]}
            database[name]['cyclohexane_DC'] = {'ref':[reference], 'LogP_exp':[LogP], 'LogP_calc': [None, None], 'energy': [None, None]}
            database[name]['water'] = {'energy': [None, None]}
            database[name]['water_DC'] = {'energy': [None, None]}
        #If it is already in database, then just add data
        #NOTE: this means we are dropping ID numbers for multiples, i.e. each compound will have the "number" it occurs under first. Which corresponds to the first time that solute appeared with a cyclohexane partition coefficient in the Leo et al. 1971 paper. 
        else:
            database[name]['cyclohexane']['ref'].append(reference)
            database[name]['cyclohexane_DC']['ref'].append(reference)
            database[name]['cyclohexane']['LogP_exp'].append(LogP)
            database[name]['cyclohexane_DC']['LogP_exp'].append(LogP)

    #If the solvent is octanol, add data to database
    elif solvent.lower() == 'octanol':
        if not database[name].has_key('octanol'):
            database[name]['octanol'] = {'ref':[reference], 'LogP_exp':[LogP]}
            database[name]['octanol_DC'] = {'ref':[reference], 'LogP_exp':[LogP]}
        else:
            database[name]['octanol']['ref'].append(reference)
            database[name]['octanol_DC']['ref'].append(reference)
            database[name]['octanol']['LogP_exp'].append(LogP)
            database[name]['octanol_DC']['LogP_exp'].append(LogP)
    # else: There should be no other options for solvent

#================================================================================================
# Now use openeye to parse names of molecules and create SMILE strings


#Adjust settings for omega so we can create random stereochemistry when it isn't specified
omega = OEOmega()
omega.SetMaxConfs(1)
omega.SetStrictStereo(False)
omega.SetStrictAtomTypes(False)

#Loop through and try and parse names with OpenEye's toolkit. 
# When they can be parsed, generate a smile string and save to key 'smile' and string of IUPAC name to 'iupac'
for molname in database.keys():
    #Create new OEChem mol to hold molecule
    mol = OEMol()

    #Try and parse IUPAC name, this should always be true!
    status = OEParseIUPACName(mol, molname)
    #Add the smile string and IUPAC name to dictionary
    if status:
        omega(mol)
        database[molname]['smile'] = OEMolToSmiles(mol)
        #Also save IUPAC name so we can change keys later if need be. 
        database[molname]['iupac'] = OECreateIUPACName(mol)

    # For future formatting it is better if there are Nones at each key instead of the dictionary not having a key
    # And there are a few solutes with no octanol literature data
    if not database[molname].has_key('octanol'):
        print database[molname]['number']
        database[molname]['octanol'] = {'ref':[None], 'LogP_exp':[None], 'energy': [None, None], 'LogP_calc': [None, None] }
        database[molname]['octanol_DC'] = {'ref':[None], 'LogP_exp':[None], 'energy': [None, None], 'LogP_calc': [None, None] }
    # else: There should be no other options for solvent


pickle.dump( database, open( "../Data/dictionary_noResults.p", "wb"))
