# Written by Caitlin C. Bannan
# Winter 2016
# Uses data in a pickled dictionary to make tables of solvation free energy and partition coefficient data
# If I were to rewrite it I would make lists of strings to then write to the file instead of printing to it
import pickle
from operator import itemgetter, attrgetter
import numpy as np

# Make all tex file lists
en = []
enDC = []
cy = []
oc = []

# Make all csv file lists
enCSV = []
enDCCSV = []
cyCSV = []
ocCSV = []

# Load database
database = pickle.load(open('../Data/dictionary_allResults.p','rb'))
xLogPdict = pickle.load(open('../Data/xLogPdata.p','rb'))

# Add initial header to table
for doc in [en, enDC, cy, oc]:
    doc.append( "\\begin{table}\n")
    doc.append("\\small\n")

en.append("\\begin{tabular}{| c | c | c | c | c |}\n")
enDC.append("\\begin{tabular}{| c | c | c | c | c |}\n")
cy.append("\\begin{tabular}{| c | c | c | c | c |}\n")
oc.append("\\begin{tabular}{| c | c | c | c | c | c |}\n")

en.append("\\hline\n")
enDC.append("\\hline\n")
cy.append("\\hline\n")
oc.append("\\hline\n")

# Label columns
en.append(" && \multicolumn{3}{ c |}{Solvation Free Energy (kJ/mol)}\\\\\n")
enDC.append(" && \multicolumn{3}{ c |}{Solvation Free Energy (kJ/mol)}\\\\\n")
en.append("DB num. & Name & Water & Cyclohexane & Octanol\\\\\n")
enDC.append("DB num. & Name & Water & Cyclohexane & Octanol\\\\\n")

enCSV.append("# Solvation Free Energies (kJ/mol) with GAFF\n")
enDCCSV.append("# Solvation Free Energies (kJ/mol) with GAFF-DC\n")

enCSV.append("DB number, Name, SMILES string, dG water, ddG water, dG cyclohexane, ddG cyclohexane, dG octanol, ddG octanol\n") 
enDCCSV.append("DB number, Name, SMILES string, dG water, ddG water, dG cyclohexane, ddG cyclohexane, dG octanol, ddG octanol\n") 

cy.append("&& \multicolumn{3}{ c |}{Cyclohexane Partition Coefficients}\\\\\n") 
cy.append("DB num. & Name & GAFF & GAFF-DC & Literature \\\\\n")
oc.append("&& \multicolumn{4}{ c |}{Octanol Partition Coefficient}\\\\\n") 
oc.append("DB num. & Name & GAFF & GAFF-DC & OExLogP & Literature \\\\\n")


cyCSV.append("# Cylohexane/water partition coefficients reported as logP\n")
ocCSV.append("# Octanol/water partition coefficients reported as logP\n")

cyCSV.append("DB number, Name, SMILES string, logP GAFF, dlogP GAFF, logP GAFF-DC, dlogP GAFF-DC, experimental logP\n")
ocCSV.append("DB number, Name, SMILES string, logP GAFF, dlogP GAFF, logP GAFF-DC, dlogP GAFF-DC, OExLogP, experimental logP\n")

en.append("\\hline\n")
enDC.append("\\hline\n")
cy.append("\\hline\n")
oc.append("\\hline\n")

# Sort keys by number order 
keys = [k for k in database.keys() if k != 'TestKeys']
nums = [int(database[k]['number']) for k in keys]
temp = zip(keys, nums)
temp = sorted(temp, key = itemgetter(1))

# define labels for octanol partition data that came from other sources
supers = {'EQ': '\\textsuperscript{\\emph{a}}', 'Sa': '\\textsuperscript{\\emph{b}}'}

for k, num in temp:
    # Get Data for GAFF energy Table
    e = database[k]
    if num == 'str': # Test Case that shouldn't be in dictionary anymore
        print num
        continue

    # Get solvation free Energy for cyclohexane with GAFF 
    cycEn = e['cyclohexane']['energy']
    cycEnergy = cycEn
    cycEn = "$%.1f \\pm %.1f$" % (cycEn[0], cycEn[1])

    # Get solvaiton free Energy for octanol with GAFF
    octEn = e['octanol']['energy']
    octEnergy = octEn
    if octEn[1] == None:
        octEn = ""
        print num, "missing calculated octanol energy"
    elif octEn[1] >= 0.5:
        octEn = "$%.0f \\pm %.0f$" % (octEn[0], octEn[1])
    else:
        octEn = "$%.1f \\pm %.1f$" % (octEn[0], octEn[1])

    # Get solvation free Energy for water with GAFF
    watEnergy = e['water']['energy']
    if watEnergy[1] >= 0.05:
        watEn = "$%.1f \\pm %.1f$" % (watEnergy[0], watEnergy[1])
    else:
        watEn = "$%.2f \\pm %.2f$" % (watEnergy[0], watEnergy[1])

    # Print Line to tex file
    en.append("%s & %s & %s & %s & %s \\\\\n" %(num, k.lower(), watEn, cycEn, octEn))
    en.append("\\hline\n")

    # Print Line to csv file
    enCSV.append("%s, %s, %s, %.2f, %.2f, %.2f, %.2f, %.2f, %.2f\n" %(num, k.lower(), e['smile'], watEnergy[0], watEnergy[0], cycEnergy[0], cycEnergy[1], octEnergy[0], octEnergy[1]))

    # REPEAT for GAFF_DC energy Table 
    cycEn = e['cyclohexane_DC']['energy']
    cycEnergy = cycEn
    if cycEn[1] == None:
        cycEn = ""
    else:
        cycEn = "$%.1f \\pm %.1f$" % (cycEn[0], cycEn[1])

    octEn = e['octanol_DC']['energy']
    octEnergy = octEn
    if octEn[1] == None:
        octEn = ""
    elif octEn[1] >= 0.5:
        octEn = "$%.0f \\pm %.0f$" % (octEn[0], octEn[1])
    else:
        octEn = "$%.1f \\pm %.1f$" % (octEn[0], octEn[1])

    watEn = e['water_DC']['energy']
    watEnergy = watEn
    if watEn[1] == None:
        watEn = ""
    else:
        watEn = "$%.1f \\pm %.1f$" % (watEn[0], watEn[1])
    # Print to file
    enDC.append("%s & %s & %s & %s & %s \\\\\n" %(num, k.lower(), watEn, cycEn, octEn))
    enDC.append("\\hline\n")
    enDCCSV.append("%s, %s, %s,  %.2f, %.2f, %.2f, %.2f, %.2f, %.2f\n" %(num, k.lower(), e['smile'], watEnergy[0], watEnergy[0], cycEnergy[0], cycEnergy[1], octEnergy[0], octEnergy[1]))
    
    # Get data for cyclohexane logP table
    c = e['cyclohexane']['LogP_calc']
    clogP = c
    c = "$%.2f \\pm %.2f$" % (c[0], c[1])

    cD = e['cyclohexane_DC']['LogP_calc']
    cDlogP = cD
    if cD[1] == None:
        cD = ''
    elif cD[1] >= 0.05:
        cD = "$%.1f \\pm %.1f$" % (cD[0], cD[1])
    else:
        cD = "$%.2f \\pm %.2f$" % (cD[0], cD[1])
        
    ex = "$%.2f$" % e['cyclohexane']['LogP_exp'][0]

    if num == 1307 or num == 1879:
        cy.append("\\textbf{%s} & \\textbf{%s} & \\textbf{%s} & \\textbf{%s} & \\textbf{%s} \\\\\n" %(num, k.lower(), c, cD, ex))
    else:
        cy.append("%s & %s & %s & %s & %s \\\\\n" %(num, k.lower(), c, cD, ex))
    cyCSV.append("%s, %s, %s, %.2f, %.2f, %.2f, %.2f, %.2f\n" % (num, k.lower(), e['smile'], clogP[0], clogP[1], cDlogP[0], cDlogP[1], e['cyclohexane']['LogP_exp'][0]))
    cy.append("\\hline\n")

    # Get data for octanol logP table
    c = e['octanol']['LogP_calc']
    clogP = c
    if c[1] == None:
        c = ''
    elif c[1] >= 0.5:
        c = "$%.1f \\pm %.1f$" % (c[0], c[1])
    else:
        c = "$%.2f \\pm %.2f$" % (c[0], c[1])

    cD = e['octanol_DC']['LogP_calc']
    cDlogP = cD
    if cD[1] == None:
        cD = ''
    elif cD[1] >= 0.05:
        cD = "$%.1f \\pm %.1f$" % (cD[0], cD[1])
    else:
        cD = "$%.2f \\pm %.2f$" % (cD[0], cD[1])
        
    ex = e['octanol']['LogP_exp'][0]
    if ex == None:
        ex = ""
        print num, 'no logP_exp in octanol'
    else:
        ex = "$%.2f$" % ex

    ref = e['octanol']['ref'][0]
    if ref == None:
        supers = ''
    elif ref[:2] == 'EQ':
        supers = '\\textsuperscript{\\emph{a}}'
    elif ref[:2] == 'Sa':
        supers = '\\textsuperscript{\\emph{b}}'
    else:
        supers = ''

    xlogp = "%.2f" % xLogPdict[str(num)]
    if num == 5785:
        oc.append("\\textbf{%s} & \\textbf{%s} & \\textbf{%s} & \\textbf{%s} & \\textbf{%s} & \\textbf{%s%s} \\\\\n" %(num, k.lower(), c, cD, xlogp, ex, supers))
    else:
        oc.append("%s & %s & %s & %s & %s & %s%s \\\\\n" %(num, k.lower(), c, cD, xlogp, ex, supers))
    ocCSV.append("%s, %s, %s, %.2f, %.2f, %.2f, %.2f, %s, %s\n" % (num, k.lower(), e['smile'], clogP[0], clogP[1], cDlogP[0], cDlogP[1], xlogp, str(e['octanol']['LogP_exp'][0])))
    oc.append("\\hline\n")

en.append("\\end{tabular}\n")
enDC.append("\\end{tabular}\n")
cy.append("\\end{tabular}\n")
oc.append("\\end{tabular}\n")

cap_GAFF = 'Complete list of calculated solvation free energies in kJ/mol for all solutes in water, cyclohexane, and octanol using GAFF'
cap_GAFF_DC = 'Complete list of calculated solvation free energies in kJ/mol for all solutes in water, cyclohexane, and octanol using GAFF-DC'
cap_cyc = 'Complete list of calculated and experimental $\\log P_{cyc}$ in GAFF and GAFF-DC where applicable. There are blanks for non-alcohols in GAFF-DC as the change in parameters only affects molecules with hydroxyl groups. Outliers by more than 3 log units are indicated in bold'
cap_oct = 'Complete list of calculated and experimental $\\log P_{oct}$ in GAFF and GAFF-DC. A blank indicates the data is unavailable, for 5 solutes experimental data for $\\log P_{oct}$ has not been located. Outliers by more than 3 log units are indicated in bold'

captions = [cap_GAFF, cap_GAFF_DC, cap_cyc, cap_oct]
labels = ['energyTableGAFF', 'energyTableGAFF-DC', 'logPcycTable','logPoctTable']

oc.append('\\textsuperscript{\\emph{a}} experimental $\\log P_{oct}$ from Corwin Hansch et al. \\textit{Exploring QSAR: Hydrophobic, Electronic, and Steric Constants} \\textbf{1995.}\n')
oc.append('\\textsuperscript{\\emph{b}} experimental $\\log P_{oct}$ from James Sangster \\textit{J. Phys. Chem. Ref. Data} \\textbf{1989,} 18, 111-1227.\n')
oc.append('All other experimental $\\log P_{oct}$ are from Albert Leo et al. \\textit{Chem. Rev.} \\textbf{1971}, 71, 525-616.\n')
cy.append('All experimental $\\log P_{cyc}$ are from Albert Leo et al. \\textit{Chem. Rev.} \\textbf{1971,} 71, 525-616.\n')

for i, doc in enumerate([en, enDC, cy, oc]):
    doc.append("\\caption{%s}\n" % captions[i])
    doc.append("\\label{%s}\n" % labels[i])
    doc.append("\\end{table}\n")

#en = open('../../extraFigures/BigTables/energyTableGAFF.tex','w')
#enDC = open('../../extraFigures/BigTables/energyTableGAFF_DC.tex','w')
#cy = open('../../extraFigures/BigTables/logPcycTable.tex','w')
#oc = open('../../extraFigures/BigTables/logPoctTable.tex', 'w')

# write csv files 
f_enCSV = open('../Data/energyTableGAFF.csv','w')
f_enCSV.writelines(enCSV)
f_enDCCSV = open('../Data/energyTableGAFF_DC.csv','w')
f_enDCCSV.writelines(enDCCSV)
f_cyCSV = open('../Data/logPcycTable.csv','w')
f_cyCSV.writelines(cyCSV)
f_ocCSV = open('../Data/logPoctTable.csv', 'w')
f_ocCSV.writelines(ocCSV)

f_enCSV.close()
f_enDCCSV.close()
f_cyCSV.close()
f_ocCSV.close()

# add tex lines to one tex file to make pdf of tables
dfile = open('../extraFigures/BigTables/2dtable.tex', 'r')
dtable = dfile.readlines()
dfile.close()


f = open('../extraFigures/BigTables/Tables.tex','w')
allLines = ['\\documentclass{article}\n', '\\usepackage[margin=0.5in]{geometry}\n', '\\usepackage{graphicx}\n', '\\begin{document}\n']
allLines += en + ['\n\n'] + enDC + ['\n\n'] + cy + ['\n\n'] + oc + ['\n\n'] + dtable + ['\n\n']   
allLines += ['\\end{document}\n']
f.writelines(allLines)
f.close()
