from solvated_mixtures import *
import numpy as np
import pickle
import os
import sys
import commands
from MDPandRunMethods import *
from hydroxynator import *

"""
This script was used to set up solvation free energy simulations in octanol and cyclohexane. 
"""

MDP_path = "/work/cluster/bannanc/PartitionCoeff/RoundThree/MDP_Path/"
big_path = "/work/cluster/bannanc/PartitionCoeff/RoundThree/%s/%s/"
#Import dictionary
# use key (number) as label for solutes and [number]['smile'] for smile string
dictionary = "dictionary_noResults.p"
database = pickle.load(open(dictionary,"rb"))
#Number of solutes
Nsolu = 1

# Label and smile string for only solvent
solvent_labels = ["octanol","cyclohexane"]
solvent_smiles = ["CCCCCCCCO", "C1CCCCC1"]

# Number of solvents needed to get to 3000 atoms
Nsolvs = [100, 150] 

current = os.getcwd()
#Construct systems for both solvents
for i, solv in enumerate(solvent_labels):
    solv_smile = solvent_smiles[i]
    Nsolv = Nsolvs[i]
    print 'Solvent: ', solv
    # For each molecule in the database create input files
    for molname, entry in database.items():
        if molname == "TestKeys": # This is a key I added to my dictionary to help with organizing at one point
            continue
        num = entry['number']
        print "\tSolute: ", num, ',', molname
        path = big_path % (solv, num)
        # Use MixtureSystem (from solvated_mixtures) to build system (Solvation Toolkit)
        builder = MixtureSystem( [num, solv], [entry['smile'], solv_smile], [Nsolu, Nsolv], path+'/')
        builder.run( just_build = True )
        builder.convert_to_gromacs()

        # Default file names created by Solvation Toolkit
        long_file = num+'_'+solv+'_1_'+str(Nsolv)

        # Move gro and top files to calculation directory
        # Also rename, because long_file name is too big to work with.
        for end in ['gro','top']:
            print commands.getoutput("cp %s/gromacs/%s.%s %s/mol.%s" % (path, long_file, end, path, end))

        # Use MDPandRunMethods to make 20 copies of each mdp file and run file
        getFiles(MDP_path, path, jobName = num+'_'+solv[:3])
        os.chdir(path)

        # Start simulation
        commands.getoutput("for ((n=0; n<20; n++)) do sbatch run.${n}.sh; done")
        os.chdir(current)

        # When solvent or solute has -OH groups use hydroxynator to apply GAFF-DC
        if entry['functGroups']['alcohol'] or solv == 'octanol':
            # Create new directory for GAFF-DC data
            oh_place_path = big_path % (solv, num+'_OH')
            os.mkdir(oh_place_path)

            # Copy topology and coordinate files for that molecule with GAFF
            print commands.getoutput("cp %s/mol* %s" % (path, oh_place_path))
            # Use MDPandRunMethods to get mdp files and run script
            getFiles(MDP_path, oh_place_path, jobName = num+'OH_'+solv[:3])
            os.chdir(oh_place_path)
            # Use the Hydroxynator to change the GAFF topology file to GAFF-DC
            hydroxynate('mol.top')

            # Start simulation on green planet cluster for each lambda value
            commands.getoutput("for ((n=0; n<20; n++)) do sbatch run.${n}.sh; done") 
            os.chdir(current)

