# Written by Caitlin C Bannan
# Mobley Group, University of California Irvine
# June 2016
"""
This script uses OpenEye's xLogP tool to calculate logPoct/wat and compare with experiment and the calculated logP values 
"""

import pickle
import sys
import numpy as np
import tools
from openeye.oechem import *
from openeye.oemolprop import *
from openeye.oeiupac import *

# Load old dictionary
database = pickle.load(open('../Data/dictionary_allResults.p','rb'))

# Store data
xlogp = []
calc = []
dcalc = []
exp = []
dexp = []

xlogPdict = {}
output = ['number,exp,xlogP,calc\n']
print "number, exp, xlogP, calc"
# Using only one prediction so I'm calling it e to match the old code:
for name, entry in database.items():
    num = entry['number']
    smile = entry['smile']
    # Generate OEMol
    mol = OEMol()
    OEParseSmiles(mol, smile)
    #Get xlogP
    x = OEGetXLogP(mol)
    xlogPdict[num] = x

    # Get calculated and exp value for this number
    c = entry['octanol']['LogP_calc'][0]
    e =entry['octanol']['LogP_exp'][0] 


    # skip numbers without octanol data
    if entry['octanol']['LogP_exp'][0] == None:
        output.append('%s,NONE,%.4f,%.4f' % (num, x, c))
        continue
    output.append('%s,%.3f,%.4f,%.4f' % (num, e, x, c))

    # Compare xlogP and calculated to experiment
    cdif = abs(c-e)
    xdif = abs(x-e)
    if xdif > cdif and abs(xdif-cdif) > 0.3:
        print "%s, \t %.2f, \t\t %.3f(%.3f), \t %.3f(%.3f)" % (num, e, x, xdif, c, cdif) 

    # Store data:
    xlogp.append(x)
    calc.append(c)
    dcalc.append(entry['octanol']['LogP_calc'][1])
    exp.append(e)
    dexp.append(0.3)

fileN = open('../Data/xLogPdata.csv','w')
fileN.writelines(output)
fileN.close()
# Save as numpy arrays
xlogp = np.array(xlogp)
calc = np.array(calc)
dcalc = np.array(dcalc)
exp = np.array(exp)
dexp = np.array(dexp)

# Error analysis for XlogP
bootits = 1000
xlogPdict['ASE'], xlogPdict['RMSE'], xlogPdict['AUE'], xlogPdict['tau'], xlogPdict['R'], xlogPdict['maxE'], xlogPdict['Correct Sign'], xlogPdict['R2'] = tools.stats_array(xlogp, exp, dexp, bootits)

for met in ['R', 'RMSE', 'ASE', 'Correct Sign']:
    print "%s, \t %.3f +/- %.3f" % (met, xlogPdict[met][0], xlogPdict[met][1]) 

pickle.dump(xlogPdict, open("../Data/xLogPdata.p", 'wb'))
